@@ -58,18 +58,26 @@ ArrayVector ZerosFunction(int, const ArrayVector& arg);
 ArrayVector DispFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector LengthFunction(int, const ArrayVector& arg);
 ArrayVector QRDFunction(int, const ArrayVector& arg);
-ArrayVector EigFunction(int, const ArrayVector& arg);
+ArrayVector EigFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector EigsFunction(int, const ArrayVector& arg);
 ArrayVector SVDFunction(int, const ArrayVector& arg);
 ArrayVector DiagFunction(int, const ArrayVector& arg);
 ArrayVector IsEmptyFunction(int, const ArrayVector& arg);
 ArrayVector ErrorFunction(int, const ArrayVector& arg);
+ArrayVector DiaryFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector WarningFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector LasterrFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector SaveFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector LoadFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector ChangeDirFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector ListFilesFunction(int, const ArrayVector& arg, Interpreter* eval);
+ArrayVector DirFunction(int, const ArrayVector& arg, Interpreter* eval);
+ArrayVector DoCLIFunction(int, const ArrayVector& arg, Interpreter* eval);
+ArrayVector RMDirFunction(int, const ArrayVector& arg);
+ArrayVector DeleteFunction(int, const ArrayVector& arg);
+ArrayVector CopyFileFunction(int, const ArrayVector& arg);
+ArrayVector MKDirCoreFunction(int, const ArrayVector& arg);
+ArrayVector FilePartsFunction(int, const ArrayVector& arg);
 ArrayVector SystemFunction(int, const ArrayVector& arg);
 ArrayVector PrintWorkingDirectoryFunction(int, const ArrayVector& arg);
 ArrayVector DirSepFunction(int, const ArrayVector& arg);
@@ -95,11 +103,13 @@ ArrayVector MinFunction(int, const ArrayVector& arg);
 ArrayVector MaxFunction(int, const ArrayVector& arg);
 ArrayVector SumFunction(int, const ArrayVector& arg);
 ArrayVector CumsumFunction(int, const ArrayVector& arg);
+ArrayVector CumprodFunction(int, const ArrayVector& arg);
 ArrayVector MeanFunction(int, const ArrayVector& arg);
 ArrayVector VarFunction(int, const ArrayVector& arg);
 ArrayVector ProdFunction(int, const ArrayVector& arg);
 ArrayVector ReshapeFunction(int, const ArrayVector& arg);
 ArrayVector PrintfFunction(int, const ArrayVector& arg, Interpreter* eval);
+ArrayVector VerStringFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector SprintfFunction(int, const ArrayVector& arg);
 ArrayVector FprintfFunction(int, const ArrayVector& arg);
 ArrayVector FscanfFunction(int, const ArrayVector& arg);
@@ -112,6 +122,7 @@ ArrayVector Arctan2Function(int, const ArrayVector& arg);
 ArrayVector CscFunction(int, const ArrayVector& arg);  
 ArrayVector SecFunction(int, const ArrayVector& arg);  
 ArrayVector WhoFunction(int, const ArrayVector& arg, Interpreter* eval);
+ArrayVector SimKeysFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector ExistFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector IsSetFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector ClearFunction(int, const ArrayVector& arg, Interpreter* eval);
@@ -122,6 +133,8 @@ ArrayVector EvalFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector EvalInFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector AssignInFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector SourceFunction(int, const ArrayVector& arg, Interpreter* eval);
+ArrayVector PauseFunction(int, const ArrayVector& arg, Interpreter* eval);
+ArrayVector ErrorCountFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector ConnectFunction(int, const ArrayVector& arg);  
 ArrayVector AcceptFunction(int, const ArrayVector& arg);  
 ArrayVector ServerFunction(int, const ArrayVector& arg);  
@@ -149,7 +162,7 @@ ArrayVector CeilFunction(int, const ArrayVector& arg);
 ArrayVector FloorFunction(int, const ArrayVector& arg);
 ArrayVector RoundFunction(int, const ArrayVector& arg);
 ArrayVector AbsFunction(int, const ArrayVector& arg);
-ArrayVector FindFunction(int, const ArrayVector& arg);
+ArrayVector FindFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector HelpFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector HelpWinFunction(int, const ArrayVector& arg, Interpreter* eval);
 ArrayVector EditorFunction(int, const ArrayVector& arg, Interpreter* eval);
@@ -194,10 +207,24 @@ ArrayVector ComputerFunction(int nargout, const ArrayVector& arg);
 ArrayVector StrStrFunction(int nargout, const ArrayVector& arg);
 ArrayVector CellFunction(int nargout, const ArrayVector& arg);
 ArrayVector StrRepStringFunction(int nargout, const ArrayVector& arg);
+ArrayVector PermuteFunction(int nargout, const ArrayVector& arg);
 ArrayVector Conv2Function(int nargout, const ArrayVector& arg);
-ArrayVector InvFunction(int nargout, const ArrayVector& arg);
+ArrayVector InvFunction(int nargout, const ArrayVector& arg, Interpreter* eval);
 ArrayVector RcondFunction(int nargout, const ArrayVector& arg);
 ArrayVector EndFunction(int nargout, const  ArrayVector& arg);
+ArrayVector XMLReadFunction(int nargout, const ArrayVector& arg);
+ArrayVector HTMLReadFunction(int nargout, const ArrayVector& arg);
+ArrayVector URLWriteFunction(int nargout, const ArrayVector& arg);
+ArrayVector RPCInitFunction(int nargout, const ArrayVector& arg);
+ArrayVector RPCIdFunction(int nargout, const ArrayVector& arg);
+ArrayVector RPCRegFunction(int nargout, const ArrayVector& arg);
+ArrayVector RPCPutFunction(int nargout, const ArrayVector& arg);
+ArrayVector RPCGetFunction(int nargout, const ArrayVector& arg);
+ArrayVector TrueFunction(int nargout, const ArrayVector& arg);
+ArrayVector FalseFunction(int nargout, const ArrayVector& arg);
+ArrayVector RegExpRepDriverFunction(int nargout, const ArrayVector& arg);
+ArrayVector RegExpFunction(int nargout, const ArrayVector& arg);
+ArrayVector RegExpIFunction(int nargout, const ArrayVector& arg);
 void Tokenize(const std::string& str, std::vector<std::string>& tokens,
 	      const std::string& delimiters = " \n");
 bool inBundleMode();
