@@ -24,27 +24,10 @@
 #include "IEEEFP.hpp"
 #include "Sparse.hpp"
 #include "helpwidget.hpp"
-#include <stdio.h>
-#include <iostream>
 #include <algorithm>
 #include "Editor.hpp"
 #include "PathTool.hpp"
 #include <QtCore>
-#include <sys/types.h>
-#include <sys/stat.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
-#include <qapplication.h>
-#include <QDir>
-#ifdef WIN32
-#include <direct.h>
-#define getcwd _getcwd
-//#define S_ISDIR(x) (x & _S_IFDIR)
-#define P_DELIM ";"
-#else
-#define P_DELIM ":"
-#endif
 #include <QtGui>
 #include "Module.hpp"
 
@@ -115,11 +98,10 @@ ArrayVector DbAutoFunction(int nargout, const ArrayVector& arg, Interpreter* eva
   } else {
     if (!arg[0].isString())
       throw Exception("dbauto function takes only a single, string argument");
-    char *txt;
-    txt = arg[0].getContentsAsCString();
-    if (strcmp(txt,"on") == 0)
+    string txt = arg[0].getContentsAsStringUpper();
+    if (txt == "ON")
       eval->AutoStop(true);
-    else if (strcmp(txt,"off") == 0)
+    else if (txt == "OFF")
       eval->AutoStop(false);
     else
       throw Exception("dbauto function argument needs to be 'on/off'");
@@ -149,7 +131,7 @@ ArrayVector HelpWinFunction(int nargout, const ArrayVector& arg, Interpreter* ev
     dir = QDir(QString(qApp->applicationDirPath() + "/../Resources/help/html"));
   } else {
     QSettings settings("FreeMat","FreeMat");
-    dir = QDir(QString(settings.value("root").toString())+"/help/html");
+    dir = QDir(QString(settings.value("root", RESOURCEDIR).toString())+"/help/html");
   }
   HelpWindow *m_helpwin = new HelpWindow(dir.canonicalPath());
   m_helpwin->show();
@@ -230,26 +212,25 @@ ArrayVector HelpFunction(int nargout, const ArrayVector& arg, Interpreter* eval)
   if (arg.size() != 1)
     throw Exception("help function requires a single argument (the function or script name)");
   Array singleArg(arg[0]);
-  char *fname;
-  fname = singleArg.getContentsAsCString();
+  string fname = singleArg.getContentsAsString();
   bool isFun;
   FuncPtr val;
   isFun = eval->getContext()->lookupFunction(fname,val);
   if (isFun && (val->type() == FM_M_FUNCTION)) {
     MFunctionDef *mptr;
     mptr = (MFunctionDef *) val;
-    mptr->updateCode();
+    mptr->updateCode(eval);
     for (int i=0;i<mptr->helpText.size();i++)
-      eval->outputMessage(mptr->helpText[i].c_str());
+      eval->outputMessage(mptr->helpText[i]);
     return ArrayVector();
   } else {
     // Check for a mdc file with the given name
     std::string mdcname;
-    mdcname = std::string(fname) + ".mdc";
+    mdcname = fname + ".mdc";
     try {
       mdcname = psearch.ResolvePath(mdcname);
     } catch (Exception& e) {
-      throw Exception(std::string("no help available on ") + fname);
+      throw Exception("no help available on " + fname);
     }
     FILE *fp;
     fp = fopen(mdcname.c_str(),"r");
@@ -280,7 +261,7 @@ ArrayVector HelpFunction(int nargout, const ArrayVector& arg, Interpreter* eval)
 	  tsize = tokens[tokencount].size();
 	  if ((outlen == 0) || ((outlen + tsize) < outputWidth)) {
 	    // Yes... send it and move on
-	    eval->outputMessage(tokens[tokencount].c_str());
+	    eval->outputMessage(tokens[tokencount]);
 	    eval->outputMessage(" ");
 	    outlen += tokens[tokencount++].size()+1;
 	  } else {
@@ -326,6 +307,11 @@ ArrayVector HelpFunction(int nargout, const ArrayVector& arg, Interpreter* eval)
 //@[
 //   clear 'global'
 //@]
+//You can use
+//@[
+//   clear 'classes'
+//@]
+//to clear all definitions of user-defined classes.
 //With no arguments, @|clear| defaults to clearing @|'all'|.
 //@@Example
 //Here is a simple example of using @|clear| to delete a variable.  First, we create a variable called @|a|:
@@ -387,6 +373,8 @@ ArrayVector ClearFunction(int nargout, const ArrayVector& arg, Interpreter* eval
       ClearPersistent(eval);
     else if (names[i] == "global")
       ClearGlobal(eval);
+    else if (names[i] == "classes")
+      eval->clearUserClasses();
     else 
       ClearVariable(eval,names[i]);
   }
@@ -443,18 +431,19 @@ ArrayVector WhoFunction(int nargout, const ArrayVector& arg, Interpreter* eval)
   } else {
     for (i=0;i<arg.size();i++) {
       Array varName(arg[i]);
-      names.push_back(varName.getContentsAsCString());
+      names.push_back(varName.getContentsAsString());
     }
   }
   std::sort(names.begin(),names.end());
   sprintf(buffer,"  Variable Name      Type   Flags             Size\n");
   eval->outputMessage(buffer);
   for (i=0;i<names.size();i++) {
-    Array lookup, *ptr;
+    Array lookup;
+    ArrayReference ptr;
     sprintf(buffer,"% 15s",names[i].c_str());
     eval->outputMessage(buffer);
     ptr = eval->getContext()->lookupVariable(names[i]);
-    if (!ptr)
+    if (!ptr.valid())
       eval->outputMessage("   <undefined>");
     else {
       lookup = *ptr;
@@ -598,6 +587,37 @@ ArrayVector FieldNamesFunction(int nargout, const ArrayVector& arg) {
 //@<
 //size(a,2)
 //@>
+//@@Tests
+//@{ test_size1.m
+//% Check the size function with an n-dim argument & one output
+//function test_val = test_size1
+//a = [];
+//a(3,7,2) = 1.0;
+//c = size(a);
+//test_val = test(c(1) == 3) & test(c(2) == 7) & test(c(3) == 2);
+//@}
+//@{ test_size2.m
+//% Check the size function with an n-dim argument & multiple outputs
+//function test_val = test_size2
+//a = [1,2,3;4,5,6];
+//[c1,c2,c3] = size(a);
+//test_val = test(c1 == 2) & test(c2 == 3) & test(c3 == 1);
+//@}
+//@{ test_size3.m
+//% Check the size function with two arguments and one output
+//function test_val = test_size3
+//a = [1,2,3;4,5,6];
+//n = size(a,2);
+//test_val = test(n == 3);
+//@}
+//@{ test_size4.m
+//% Check the size function with two arguments and two outputs
+//function test_val = test_size4
+//a = [1,2,3;4,5,6];
+//test_val = 0;
+//[c,d] = size(a,2);
+//test_val = (c == 3);
+//@}
 //!
 ArrayVector SizeFunction(int nargout, const ArrayVector& arg) {
   ArrayVector retval;
@@ -640,7 +660,7 @@ ArrayVector SizeFunction(int nargout, const ArrayVector& arg) {
 //     return retval;
 //   }
 
-int ExistBuiltinFunction(char* fname, Interpreter* eval) {    
+int ExistBuiltinFunction(string fname, Interpreter* eval) {    
   bool isDefed;
   FuncPtr d;
   isDefed = eval->getContext()->lookupFunction(fname,d);
@@ -652,18 +672,19 @@ int ExistBuiltinFunction(char* fname, Interpreter* eval) {
   return 0;
 }
 
-int ExistDirFunction(char* fname, Interpreter* eval) {
-  struct stat filestat;
+int ExistDirFunction(string fname, Interpreter* eval) {
   // Check for extra termination
-  if ((fname[strlen(fname)-1] == '/') ||
-      (fname[strlen(fname)-1] == '\\'))
-    fname[strlen(fname)-1] = 0;
-  if (stat(fname,&filestat)==-1) return 0;
-  if (S_ISDIR(filestat.st_mode)) return 7;
+  int flen = fname.size();
+  if ((fname[flen-1] == '/') ||
+      (fname[flen-1] == '\\'))
+    fname[flen-1] = 0;
+  QFileInfo filestat(QString::fromStdString(fname));
+  if (!filestat.exists()) return 0;
+  if (filestat.isDir()) return 7;
   return 0;
 }
 
-int ExistFileFunction(char* fname, Interpreter* eval) {
+int ExistFileFunction(string fname, Interpreter* eval) {
   PathSearcher src(eval->getPath());
   try {
     src.ResolvePath(fname);
@@ -678,15 +699,15 @@ int ExistFileFunction(char* fname, Interpreter* eval) {
   return 0;
 }
 
-int ExistVariableFunction(char* fname, Interpreter* eval) {
-  bool isDefed = (eval->getContext()->lookupVariable(fname) != NULL);
+int ExistVariableFunction(string fname, Interpreter* eval) {
+  bool isDefed = (eval->getContext()->lookupVariable(fname).valid());
   if (isDefed)
     return 1;
   else
     return 0;
 }
 
-int ExistAllFunction(char* fname, Interpreter* eval) {
+int ExistAllFunction(string fname, Interpreter* eval) {
   int ret;
   ret = ExistVariableFunction(fname,eval);
   if (ret) return ret;
@@ -731,11 +752,10 @@ ArrayVector IsSetFunction(int nargout, const ArrayVector& arg, Interpreter* eval
   if (arg.size() < 1)
     throw Exception("isset function takes at least one argument - the name of the variable to check for");
   Array tmp(arg[0]);
-  char *fname;
-  fname = tmp.getContentsAsCString();
+  string fname = tmp.getContentsAsString();
   bool isDefed;
-  Array *d = eval->getContext()->lookupVariable(fname);
-  isDefed = (d != NULL);
+  ArrayReference d = eval->getContext()->lookupVariable(fname);
+  isDefed = (d.valid());
   if (isDefed && !d->isEmpty())
     return singleArrayVector(Array::logicalConstructor(1));
   else
@@ -799,30 +819,52 @@ ArrayVector IsSetFunction(int nargout, const ArrayVector& arg, Interpreter* eval
 //exist('b')
 //exist('c')
 //@>
+//@@Tests
+//@{ test_exist1.m
+//function x = test_exist1
+//x = test_exist1_assist;
+//x = test_exist1_assist;
+//
+//function y = test_exist1_assist
+//persistent x
+//if (exist('x'))
+//  y = 1;
+//else
+//  y = 0;
+//  x = 1;
+//end
+//@}
+//@{ test_exist2.m
+//function x = test_exist2
+//persistent y
+//x = 1;
+//if (exist('y'))
+//  x = 0;
+//end
+//@}
 //!
 ArrayVector ExistFunction(int nargout, const ArrayVector& arg, Interpreter* eval) {
   if (arg.size() < 1)
     throw Exception("exist function takes at least one argument - the name of the object to check for");
   Array tmp(arg[0]);
-  char *fname;
-  fname = tmp.getContentsAsCString();
-  char *stype;
+  string fname = tmp.getContentsAsString();
+  string stype;
   if (arg.size() > 1) {
     Array tmp2(arg[1]);
-    stype = tmp2.getContentsAsCString();
+    stype = tmp2.getContentsAsString();
   } else {
     stype = "all";
   }
   int retval;
-  if (strcmp(stype,"all")==0)
+  if (stype=="all")
     retval = ExistAllFunction(fname,eval);
-  else if (strcmp(stype,"builtin")==0)
+  else if (stype=="builtin")
     retval = ExistBuiltinFunction(fname,eval);
-  else if (strcmp(stype,"dir")==0)
+  else if (stype=="dir")
     retval = ExistDirFunction(fname,eval);
-  else if (strcmp(stype,"file")==0)
+  else if (stype=="file")
     retval = ExistFileFunction(fname,eval);
-  else if (strcmp(stype,"var")==0)
+  else if (stype=="var")
     retval = ExistVariableFunction(fname,eval);
   else throw Exception("Unrecognized search type for function 'exist'");
   return singleArrayVector(Array::int32Constructor(retval));
@@ -1099,8 +1141,7 @@ ArrayVector WhereFunction(int nargout, const ArrayVector& arg, Interpreter* eval
 ArrayVector WhichFunction(int nargout, const ArrayVector& arg, Interpreter* eval) {
   if (arg.size() != 1)
     throw Exception("which function takes one string argument (the name of the function to look up)");
-  char *fname;
-  fname = arg[0].getContentsAsCString();
+  string fname = arg[0].getContentsAsString();
   bool isFun;
   FuncPtr val;
   isFun = eval->getContext()->lookupFunction(fname,val);
@@ -1110,48 +1151,50 @@ ArrayVector WhichFunction(int nargout, const ArrayVector& arg, Interpreter* eval
     if (val->type() == FM_M_FUNCTION) {
       MFunctionDef *mptr;
       mptr = (MFunctionDef *) val;
-      mptr->updateCode();
+      mptr->updateCode(eval);
       if (mptr->pcodeFunction) {
 	if (mptr->scriptFlag) {
 	  if (nargout == 0) {
-	    sprintf(buffer,"Function %s, P-code script\n",fname);
+	    sprintf(buffer,"Function %s, P-code script\n",fname.c_str());
 	    eval->outputMessage(buffer);
 	  }
 	} else {
 	  if (nargout == 0) {
-	    sprintf(buffer,"Function %s, P-code function\n",fname);
+	    sprintf(buffer,"Function %s, P-code function\n",fname.c_str());
 	    eval->outputMessage(buffer);
 	  }
 	}
       } else {
 	if (mptr->scriptFlag) {
 	  if (nargout == 0) {
-	    sprintf(buffer,"Function %s, M-File script in file '%s'\n",fname,mptr->fileName.c_str());
+	    sprintf(buffer,"Function %s, M-File script in file '%s'\n",fname.c_str(),
+		    mptr->fileName.c_str());
 	    eval->outputMessage(buffer);
 	  } else 
-	    ret = Array::stringConstructor(mptr->fileName.c_str());
+	    ret = Array::stringConstructor(mptr->fileName);
 	} else {
 	  if (nargout == 0) {
-	    sprintf(buffer,"Function %s, M-File function in file '%s'\n",fname,mptr->fileName.c_str());
+	    sprintf(buffer,"Function %s, M-File function in file '%s'\n",
+		    fname.c_str(),mptr->fileName.c_str());
 	    eval->outputMessage(buffer);
 	  } else
-	    ret = Array::stringConstructor(mptr->fileName.c_str());
+	    ret = Array::stringConstructor(mptr->fileName);
 	}
       }
     } else if ((val->type() == FM_BUILT_IN_FUNCTION) || (val->type() == FM_SPECIAL_FUNCTION) ) {
       if (nargout == 0) {
-	sprintf(buffer,"Function %s is a built in function\n",fname);
+	sprintf(buffer,"Function %s is a built in function\n",fname.c_str());
 	eval->outputMessage(buffer);
       }
     } else {
       if (nargout == 0) {
-	sprintf(buffer,"Function %s is an imported function\n",fname);
+	sprintf(buffer,"Function %s is an imported function\n",fname.c_str());
 	eval->outputMessage(buffer);
       }
     }
   } else {
     if (nargout == 0) {
-      sprintf(buffer,"Function %s is unknown!\n",fname);
+      sprintf(buffer,"Function %s is unknown!\n",fname.c_str());
       eval->outputMessage(buffer);
     }
   }
@@ -1464,9 +1507,69 @@ ArrayVector FindModeSparse(Array x, int nargout) {
 //@<
 //[r,c,v] = find(A)
 //@>
+//@@Tests
+//@{ test_sparse20.m
+//function x = test_sparse20
+//a = [0,0,3,1,3;1,0,0,0,2;4,3,0,2,0];
+//[i,j,v] = find(a);
+//B = sparse(i,j,v);
+//x = testeq(a,B);
+//@}
+//@{ test_sparse21.m
+//function x = test_sparse21
+//ai = [0,2,6,0,1;3,0,3,0,2;0,0,3,0,2];
+//ar = [1,2,0,0,4;3,2,0,0,5;0,0,3,0,2];
+//a = complex(ar+i*ai);
+//[i,j,v] = find(a);
+//B = sparse(i,j,v);
+//x = testeq(a,B);
+//@}
+//@{ test_sparse68.m
+//% Test sparse to IJV conversion
+//function x = test_sparse68
+//[yi1,zi1] = sparse_test_mat('int32',300,400);
+//[yf1,zf1] = sparse_test_mat('float',300,400);
+//[yd1,zd1] = sparse_test_mat('double',300,400);
+//[yc1,zc1] = sparse_test_mat('complex',300,400);
+//[yz1,zz1] = sparse_test_mat('dcomplex',300,400);
+//[iyi1,jyi1,vyi1] = find(yi1);
+//[izi1,jzi1,vzi1] = find(zi1);
+//[iyf1,jyf1,vyf1] = find(yf1);
+//[izf1,jzf1,vzf1] = find(zf1);
+//[iyd1,jyd1,vyd1] = find(yd1);
+//[izd1,jzd1,vzd1] = find(zd1);
+//[iyc1,jyc1,vyc1] = find(yc1);
+//[izc1,jzc1,vzc1] = find(zc1);
+//[iyz1,jyz1,vyz1] = find(yz1);
+//[izz1,jzz1,vzz1] = find(zz1);
+//x = testeq(iyi1,izi1) & testeq(jyi1,jzi1) & testeq(vyi1,vzi1);
+//x = x & testeq(iyf1,izf1) & testeq(jyf1,jzf1) & testeq(vyf1,vzf1);
+//x = x & testeq(iyd1,izd1) & testeq(jyd1,jzd1) & testeq(vyd1,vzd1);
+//x = x & testeq(iyc1,izc1) & testeq(jyc1,jzc1) & testeq(vyc1,vzc1);
+//x = x & testeq(iyz1,izz1) & testeq(jyz1,jzz1) & testeq(vyz1,vzz1);
+//@}
+//@{ test_sparse70.m
+//% Test sparse to IJV to sparse conversion
+//function x = test_sparse70
+//[yi1,zi1] = sparse_test_mat('int32',300,400);
+//[yf1,zf1] = sparse_test_mat('float',300,400);
+//[yd1,zd1] = sparse_test_mat('double',300,400);
+//[yc1,zc1] = sparse_test_mat('complex',300,400);
+//[yz1,zz1] = sparse_test_mat('dcomplex',300,400);
+//[iyi1,jyi1,vyi1] = find(yi1);
+//[iyf1,jyf1,vyf1] = find(yf1);
+//[iyd1,jyd1,vyd1] = find(yd1);
+//[iyc1,jyc1,vyc1] = find(yc1);
+//[iyz1,jyz1,vyz1] = find(yz1);
+//x = testeq(sparse(iyi1,jyi1,vyi1),zi1);
+//x = x & testeq(sparse(iyf1,jyf1,vyf1),zf1);
+//x = x & testeq(sparse(iyd1,jyd1,vyd1),zd1);
+//x = x & testeq(sparse(iyc1,jyc1,vyc1),zc1);
+//x = x & testeq(sparse(iyz1,jyz1,vyz1),zz1);
+//@}
 //!  
 
-ArrayVector FindTrim(ArrayVector a, int cnt, bool first_flag) {
+ArrayVector FindTrim(ArrayVector a, int cnt, bool first_flag, Interpreter* m_eval) {
   if (cnt < 0) return a;
   if (a.size() == 0) return a;
   int N = a[0].getLength();
@@ -1479,11 +1582,11 @@ ArrayVector FindTrim(ArrayVector a, int cnt, bool first_flag) {
   else
     ndx = Array::int32RangeConstructor((N-cnt)+1,1,N,vertflag);
   for (int i=0;i<a.size();i++) 
-    ret.push_back(a[i].getVectorSubset(ndx));
+    ret.push_back(a[i].getVectorSubset(ndx,m_eval));
   return ret;
 }
 
-ArrayVector FindFunction(int nargout, const ArrayVector& arg) {
+ArrayVector FindFunction(int nargout, const ArrayVector& arg, Interpreter* m_eval) {
   // Detect the Find mode...
   if (arg.size() < 1)
     throw Exception("find function takes at least one argument");
@@ -1493,10 +1596,10 @@ ArrayVector FindFunction(int nargout, const ArrayVector& arg) {
   if (arg.size() > 1)
     k  = ArrayToInt32(arg[1]);
   if (arg.size() == 3) {
-    const char *flag = ArrayToString(arg[2]);
-    if (strcmp(flag,"first") == 0)
+    string flag = ArrayToString(arg[2]);
+    if (flag=="first")
       first_flag = true;
-    else if (strcmp(flag,"last") == 0)
+    else if (flag=="last")
       first_flag = false;
     else
       throw Exception("third option to find must be either 'first' or 'last'");
@@ -1504,14 +1607,14 @@ ArrayVector FindFunction(int nargout, const ArrayVector& arg) {
   if (tmp.isReferenceType())
     throw Exception("find does not work on reference types (cell-arrays or structure arrays)");
   if ((nargout <= 1) && !tmp.sparse())
-    return FindTrim(SingleFindModeFull(tmp),k,first_flag);
+    return FindTrim(SingleFindModeFull(tmp),k,first_flag,m_eval);
   if ((nargout == 2) && !tmp.sparse())
-    return FindTrim(RCFindModeFull(tmp),k,first_flag);
+    return FindTrim(RCFindModeFull(tmp),k,first_flag,m_eval);
   if ((nargout == 3) && !tmp.sparse())
-    return FindTrim(RCVFindModeFull(tmp),k,first_flag);
+    return FindTrim(RCVFindModeFull(tmp),k,first_flag,m_eval);
   if (nargout > 3)
     throw Exception("Do not understand syntax of find call (too many output arguments).");
-  return FindTrim(FindModeSparse(tmp,nargout),k,first_flag);
+  return FindTrim(FindModeSparse(tmp,nargout),k,first_flag,m_eval);
 }
 
 //!
