@@ -89,19 +89,28 @@ void lmdif1_(fcnptr, int*m, int*n, double*x, double*fvec, double*tol,
 //\]
 //Note that both @|x| and @|y| (and the output of the function) must all
 //be real variables.  Complex variables are not handled yet.
+//@@Tests
+//@{ test_fitfun1.m
+//% Test the fitfun bug (bug 1514605)
+//function test_val = test_fitfun1
+//O = 3/4*pi;
+//y0 = cos(O);
+//[x y] = fitfun('cos',3.5/4*pi,y0,1,0.01);
+//test_val = abs((x-O)/O*100)<.1;
+//@}
 //!
 ArrayVector FitFunFunction(int nargout, const ArrayVector& arg, Interpreter* eval) {
   if (arg.size()<4) 
     throw Exception("fitfun requires at least four arguments");
   if (!(arg[0].isString()))
     throw Exception("first argument to fitfun must be the name of a function (i.e., a string)");
-  char *fname = arg[0].getContentsAsCString();
+  string fname = arg[0].getContentsAsString();
   eval->rescanPath();
   Context *context = eval->getContext();
-  FunctionDef *funcDef;
+  FuncPtr funcDef;
   if (!context->lookupFunction(fname,funcDef))
     throw Exception(std::string("function ") + fname + " undefined!");
-  funcDef->updateCode();
+  funcDef->updateCode(eval);
   if (funcDef->scriptFlag)
     throw Exception("cannot use feval on a script");
   a_funcDef = funcDef;
