@@ -29,25 +29,17 @@
 #include "RefVec.hpp"
 #include <QVector>
 #include <QList>
+#include <QStringList>
 #include <QSharedDataPointer>
 
 class Array;
 class Interpreter;
 
-//typedef std::vector<Array> ArrayVector;
-//typedef RefVec<Array> ArrayVector;
-
-//typedef QVector<Array> ArrayVector;
 typedef QList<Array> ArrayVector;
+ArrayVector singleArrayVector(Array);
 typedef QVector<ArrayVector> ArrayMatrix;
 
-//class ArrayVector;
-
-ArrayVector singleArrayVector(Array);
-
-//typedef std::vector<ArrayVector> ArrayMatrix;
-
-class FunctionDef;
+class FuncPtr;
 
 static Dimensions zeroDim(0,0);
 
@@ -137,7 +129,7 @@ public:
    * Throws an exception for string, cell, structure types, or if a zero or 
    * negative index is encountered.
    */
-  void toOrdinalType();
+  void toOrdinalType(Interpreter *m_eval);
   /**
    * Default constructor.
    */
@@ -494,7 +486,12 @@ public:
    * FM_STRUCT_ARRAY, so that the fieldNames argument is not needed.
    */
   void promoteType(Class new_type);
-
+  /**
+   * Permute our array according to the given permutation vector.  Note that
+   * we assume that the permutation vector is of the correct size and is a
+   * valid permutation.
+   */
+  void permute(const int32* permutation);
   /**
    * Diagonal constructor - construct an array from a given vector, with
    * the contents of the vector stored into the specified diagonal of the
@@ -507,7 +504,7 @@ public:
    * Empty constructor
    */
   static Array emptyConstructor();
-  static Array funcPtrConstructor(FunctionDef *fptr);
+  static Array funcPtrConstructor(FuncPtr fptr);
   /**
    * Scalar constructor - Construct an FM_LOGICAL object with a scalar
    * value.
@@ -649,6 +646,8 @@ public:
    * Throws an exception if the geometry of the argumens is incompatible.
    */
   static Array cellConstructor(ArrayMatrix& m);
+  static Array cellConstructor(ArrayVector& m);
+  static Array cellConstructor(Array m);
   /**
    * Structure constructor - this is equivalent to the built in struct command.
    * First, we have to make sure that each entry of "values" have 
@@ -673,14 +672,14 @@ public:
    *  - the variable is empty
    *  - the argument subset exceeds our valid domain
    */
-  Array getVectorSubset(Array& index);
+  Array getVectorSubset(Array& index, Interpreter* m_eval);
   /**
    * Get a subset of an Array.  This if for n-Dimensional-indexing, meaning
    * that x(10) is really x(10,1).
    * Throws an exception if the variable is empty.
    */
-  Array getNDimSubset(ArrayVector& index);
-  Array getNDimSubsetScalars(ArrayVector& index);
+  Array getNDimSubset(ArrayVector& index, Interpreter* m_eval);
+  Array getNDimSubsetScalars(ArrayVector& index, Interpreter* m_eval);
   /**
    * Get the diagonal elements of an array.  Only applicable to 2-dimensional arrays.
    * The diagonal part of a rectangular matrix
@@ -707,15 +706,15 @@ public:
    *   - we are not a cell-array
    *   - the indices exceed the array bounds
    */
-  Array getVectorContents(Array& index);
-  ArrayVector getVectorContentsAsList(Array& index);
+  Array getVectorContents(Array& index, Interpreter* m_eval);
+  ArrayVector getVectorContentsAsList(Array& index, Interpreter* m_eval);
   /**
    * Get a subset of an Array using contents-addressing.  This is used when a cell array
    * is used to supply a list of expressions.
    * Throws an exception if we are not a cell-array.
    */  
-  Array getNDimContents(ArrayVector& index);
-  ArrayVector getNDimContentsAsList(ArrayVector& index);
+  Array getNDimContents(ArrayVector& index, Interpreter* m_eval);
+  ArrayVector getNDimContentsAsList(ArrayVector& index, Interpreter* m_eval);
   void setValue(const Array &x);
   /**
    * Set a subset of an Array.  Uses vector-indexing, meaning that the
@@ -723,14 +722,14 @@ public:
    * So, x(10) is equivalent to x(:)(10), even if, say, x is 3 x 4.
    * Throws an exception if there is a size mismatch between the index and the data.
    */
-  void setVectorSubset(Array& index, Array& data);
+  void setVectorSubset(Array& index, Array& data, Interpreter* m_eval);
   /**
    * Set a subset of an Array.   This if for n-Dimensional-indexing, meaning
    * that x(10) is really x(10,1).
    * Throws an exception if there is a size mismatch between the index and the data.
    */
-  void setNDimSubset(ArrayVector& index, Array& data);
-  void setNDimSubsetScalars(ArrayVector& index, const Array& data);
+  void setNDimSubset(ArrayVector& index, Array& data, Interpreter* m_eval);
+  void setNDimSubsetScalars(ArrayVector& index, Array& data, Interpreter* m_eval);
   /**
    * Set a subset of an Array using contents-indexing, meaning that the
    * argument is assumed to refer to the elements in their order as a vector.
@@ -740,7 +739,7 @@ public:
    * Throws an exception if the number of elements in data do not match
    * the number of indices in index.
    */
-  void setVectorContentsAsList(Array& index, ArrayVector& data);
+  void setVectorContentsAsList(Array& index, ArrayVector& data, Interpreter* m_eval);
   /**
    * Set a subset of an Array.   This if for n-Dimensional-indexing, meaning
    * that x{10} is really x{10,1}.  This is used when a cell-array is used
@@ -749,7 +748,7 @@ public:
    * the number of indices covered by index (which is the product of the
    * number of elements in each dimension of index).
    */
-  void setNDimContentsAsList(ArrayVector& index, ArrayVector& data);
+  void setNDimContentsAsList(ArrayVector& index, ArrayVector& data, Interpreter* m_eval);
   /**
    * Replace the contents of a field with the supplied array.  This is used
    * when a structure array is used to hold the return of a multi-function
@@ -769,7 +768,7 @@ public:
    *   -  Copy (and skip) as necessary.
    * The result is then resized using the same rules as in vectorResize.
    */ 
-  void deleteVectorSubset(Array& ndx);
+  void deleteVectorSubset(Array& ndx, Interpreter* m_eval);
   /**
    * Delete a subset of this array using the arguments for n-Dimensional
    * indexing.  This method is the "planar" delete, meaning that its 
@@ -777,11 +776,11 @@ public:
    * It cannot be used to create "holes" in an array.
    * Throws an exception if the argument contains more than one non-colon index
    */
-  void deleteNDimSubset(ArrayVector& args);
+  void deleteNDimSubset(ArrayVector& args, Interpreter* m_eval);
   /**
    * Summarize this array when it appears in a Cell array.
    */
-  void summarizeCellEntry() const;
+  void summarizeCellEntry(Interpreter* eval) const;
   /**
    * Print some reasonable representation of this array to the
    * the supplied stream.
@@ -803,7 +802,8 @@ public:
    * Get our contents as a C-string.  Only works for _STRING types.
    * Throws an exception for non-string types.
    */
-  char* getContentsAsCString() const;
+  string getContentsAsString() const;
+  string getContentsAsStringUpper() const;
   /**
    * Get our contents as an integer scalar.
    * Throws an exception if we are not a scalar integer type.
@@ -820,14 +820,6 @@ public:
    * FM_STRUCT_ARRAY.
    */
   static bool isDataClassReferenceType(Class);
-  /**
-   * Sets the IO interface used by instances of the Array class.
-   */
-  static void setArrayInterpreter(Interpreter* eval);
-  /**
-   * Returns the IO interface being used by instances of the Array class.
-   */
-  static Interpreter* getArrayInterpreter();
   /**
    * Returns true if any of the entries in the array are not finite
    * (always false for reference and integer types)
@@ -843,7 +835,10 @@ public:
 bool isColonOperator(Array& a);
 int32 ArrayToInt32(const Array& a);
 double ArrayToDouble(const Array& a);
-const char* ArrayToString(const Array& a);
+string ArrayToString(const Array& a);
+Array ToSingleArray(const ArrayVector& a);
+Array CellArrayFromQStringList(QStringList t);
+Array Uint32VectorFromQList(QList<uint32> p);
 
 void printObjectBalance();
 void dumpAllArrays();
@@ -852,5 +847,5 @@ uint32 TypeSize(Class cls);
 
 string operator+(string a, int d);
 string operator+(int d, string a);
-
+stringVector operator+(stringVector a, stringVector b);
 #endif
