@@ -31,14 +31,76 @@
 #include <QMutex>
 #include <stdarg.h>
 
+/**
+ * This class represents a reference to an array in a scope
+ */
+class ArrayReference {
+  Array *m_ptr;
+  bool m_global;
+  Scope *m_scope;
+public:
+  inline ArrayReference() : m_ptr(NULL), m_global(false), m_scope(NULL) {
+  }
+  inline ArrayReference(Array *ptr) :  m_ptr(ptr), m_global(false), m_scope(NULL) {
+  }
+  inline ArrayReference(Array *ptr, bool global, Scope *scope) :
+    m_ptr(ptr), m_global(global), m_scope(scope) {
+    if (m_global)
+      m_scope->lock();
+  }
+  inline ~ArrayReference() {
+    if (m_global)
+      m_scope->unlock();
+  }
+  inline ArrayReference(const ArrayReference& copy) {
+    m_ptr = copy.m_ptr;
+    m_global = copy.m_global;
+    m_scope = copy.m_scope;
+    if (m_global)
+      m_scope->lock();
+  }
+  inline ArrayReference& operator=(Array* ptr) {
+    if (m_global)
+      m_scope->unlock();
+    m_ptr = ptr;
+    m_global = false;
+    m_scope = NULL;
+  }
+  inline ArrayReference& operator=(const ArrayReference& copy) {
+    if (m_global)
+      m_scope->unlock();
+    m_ptr = copy.m_ptr;
+    m_global = copy.m_global;
+    m_scope = copy.m_scope;
+    if (m_global)
+       m_scope->lock();
+    return *this;
+  }
+  inline Array& operator*() {
+    return *m_ptr;
+  }
+  inline Array* operator->() {
+    return m_ptr;
+  }
+  inline bool valid() {
+    return (m_ptr != NULL);
+  }
+};
+
 /**
  * A Context is a stack of scopes with the (peculiar) property that
  * the top and bottom of the stack (global and local scopes respectively)
  * are searched regularly.  The context is responsible for determining
  * if variables and functions exist, and if so, for returning their
  * values and accepting modifications to them.  A context also keeps
- * track of loop depth.
+ * track of loop depth.  Furthermore, the context (starting with 3.1)
+ * now "owns" all of the functions that are defined.  These used to be
+ * tracked with the scopes, but that lead to a memory leak of FunctionDef
+ * objects.
  */
+
+typedef SymbolTable<FuncPtr> CodeTable;
+
 class Context {
   /**
    * The normal stack of scopes.
@@ -48,6 +110,10 @@ class Context {
    * The stack of scopes that have been "bypassed"
    */
   std::vector<Scope*> bypassstack;
+  /**
+   * The table of functions
+   */
+  CodeTable codeTab;
   /**
    * List of functions that are "temporary" and should be flushed
    */
@@ -69,8 +135,8 @@ public:
    * Create a context and initialize it with a global scope and a 
    * base scope.
    */
-  Context() : mutex(QMutex::Recursive) {
-    pushScope("global");
+  Context(Scope *global) : mutex(QMutex::Recursive) {
+    scopestack.push_back(global);
     pushScope("base");
     topScope = scopestack.front();
     bottomScope = scopestack.back();
@@ -79,11 +145,10 @@ public:
   /**
    * Delete the context
    */
-  ~Context() {
-    while (!scopestack.empty()) {
-      delete scopestack.back();
-      scopestack.pop_back();
-    }
+  inline ~Context() {
+    // Don't delete the base scope
+    for (int i=1;i<scopestack.size();i++)
+      delete scopestack[i];
   }
 
   /**
@@ -151,7 +216,7 @@ public:
   inline void insertVariable(const std::string& varName, const Array& var) {
     Scope* active;
     std::string mapName;
-    
+
     if (bottomScope->isVariablePersistent(varName)) {
       mapName = bottomScope->getMangledName(varName);
       active = topScope;
@@ -185,20 +250,21 @@ public:
    * with the given name, and inserted into the scope that was
    * searched.  A pointer to this newly created variable is returned.
    */
-  inline Array* lookupVariable(const std::string& varName) {
+  inline ArrayReference lookupVariable(const std::string& varName) {
     Scope* active;
     std::string mapName;
-    
+    bool global = false;
     if (bottomScope->isVariablePersistent(varName)) {
       mapName = bottomScope->getMangledName(varName);
       active = topScope;
     } else if (bottomScope->isVariableGlobal(varName)) {
       mapName = varName;
       active = topScope;
-    } else {
-      return (bottomScope->lookupVariable(varName));
+      global = true;
+     } else {
+      return (ArrayReference(bottomScope->lookupVariable(varName),global,bottomScope));
     }
-    return (active->lookupVariable(mapName));
+    return (ArrayReference(active->lookupVariable(mapName),global,active));
   }
   /**
    * Look for a variable, but only locally.
@@ -207,27 +273,26 @@ public:
     return bottomScope->lookupVariable(varName);
   }
   /**
-   * Insert a function definition into the local scope (bottom of
-   * the scope stack).
+   * Insert a function definition into the code table.
    */
-  inline void insertFunctionLocally(FuncPtr f) {
-    bottomScope->insertFunction(f);
-  }
-  /**
-   * Insert a function definition into the global scope (top of the
-   * scope stack).
-   */
-  inline void insertFunctionGlobally(FuncPtr f, bool temporary) {
-    topScope->insertFunction(f);
+  inline void insertFunction(FuncPtr f, bool temporary) {
+    //    qDebug() << "inserting function " << QString::fromStdString(f->name) << "\r";
+    codeTab.insertSymbol(f->name,f);
     if (temporary)
       tempFunctions.push_back(f->name);
   }
+  /**
+   * Remove a function definition from the code table.
+   */
+  inline void deleteFunction(const std::string& funcName) {
+    codeTab.deleteSymbol(funcName);
+  }
   /**
    * Flush temporary function definitions from the global context
    */
   inline void flushTemporaryGlobalFunctions() {
     for (int i=0;i<tempFunctions.size();i++)
-      topScope->deleteFunction(tempFunctions[i]);
+      deleteFunction(tempFunctions[i]);
     tempFunctions.clear();
   }
   /**
@@ -256,10 +321,10 @@ public:
     f2def = new BuiltInFunctionDef;
     f2def->retCount = argc_out;
     f2def->argCount = argc_in;
-    f2def->name = strdup(name);
+    f2def->name = name;
     f2def->fptr = fptr;
     f2def->arguments = args;
-    topScope->insertFunction(f2def);  
+    insertFunction(f2def,false);  
   }
   /**
    * Add a special function to the global scope with the given name.
@@ -287,10 +352,10 @@ public:
     f2def = new SpecialFunctionDef;
     f2def->retCount = argc_out;
     f2def->argCount = argc_in;
-    f2def->name = strdup(name);
+    f2def->name = name;
     f2def->fptr = fptr;
     f2def->arguments = args;
-    topScope->insertFunction(f2def);
+    insertFunction(f2def,false);
   }
   /**
    * Add a built in function to the global scope with the given name
@@ -319,11 +384,11 @@ public:
     f2def = new BuiltInFunctionDef;
     f2def->retCount = argc_out;
     f2def->argCount = argc_in;
-    f2def->name = strdup(name);
+    f2def->name = name;
     f2def->fptr = fptr;
     f2def->arguments = args;
     f2def->graphicsFunction = true;
-    topScope->insertFunction(f2def);  
+    insertFunction(f2def,false);  
   }
   /**
    * Add a special function to the global scope with the given name, and
@@ -352,39 +417,38 @@ public:
     f2def = new SpecialFunctionDef;
     f2def->retCount = argc_out;
     f2def->argCount = argc_in;
-    f2def->name = strdup(name);
+    f2def->name = name;
     f2def->fptr = fptr;
     f2def->arguments = args;
     f2def->graphicsFunction = true;
-    topScope->insertFunction(f2def);
+    insertFunction(f2def,false);
   }
   
   inline stringVector listAllFunctions() {
-    stringVector retval(topScope->listAllFunctions());
-    stringVector bottomFunctions(bottomScope->listAllFunctions());
-    for (int i=0;i<bottomFunctions.size();i++)
-      retval.push_back(bottomFunctions[i]);
-    return retval;
+    return codeTab.getCompletions("");
+  }
+
+  inline stringVector getCompletions(const std::string& prefix) {
+    stringVector local_completions = bottomScope->getCompletions(prefix);
+    stringVector global_completions = topScope->getCompletions(prefix);
+    stringVector code_completions = codeTab.getCompletions(prefix);
+    stringVector completions(local_completions);
+    for (int i=0;i<global_completions.size();i++)
+      completions.push_back(global_completions[i]);
+    for (int i=0;i<code_completions.size();i++)
+      completions.push_back(code_completions[i]);
+    return completions;
   }
 
   inline bool lookupFunction(std::string funcName, FuncPtr& val) {
-    bool localFunction;
-    if (bottomScope->lookupFunction(funcName,val))
+    FuncPtr* ret = codeTab.findSymbol(funcName);
+    if (ret) {
+      val = *ret;
       return true;
-    return topScope->lookupFunction(funcName,val);
+    }
+    return false;
   }
     
-  inline bool lookupFunctionLocally(std::string funcName, FuncPtr& val) {
-    return bottomScope->lookupFunction(funcName,val);
-  }
-
-  inline bool lookupFunctionGlobally(std::string funcName, FuncPtr& val) {
-    return topScope->lookupFunction(funcName,val);
-  }
-    
-  inline void deleteFunctionGlobally(std::string funcName) {
-    topScope->deleteFunction(funcName);
-  }
   /**
    * Add a persistent variable to the local stack.  This involves
    * two steps:
