@@ -17,47 +17,35 @@
  *
  */
 
-#if HAVE_AVCALL
-#include "avcall.h"
-#endif
 #include "FunctionDef.hpp"
 #include "Interpreter.hpp"
 #include "Parser.hpp"
 #include "Exception.hpp"
-#include <stdio.h>
-#include <sys/stat.h>
-#ifndef WIN32
-#include <unistd.h>
-#endif
-#include <sys/types.h>
 #include <iostream>
 #include <signal.h>
 #include "SymbolTable.hpp"
 #include "Types.hpp"
 #include "MexInterface.hpp"
 #include <QDebug>
+#include <sys/stat.h>
 
-#ifdef WIN32
-#define snprintf _snprintf
+#if HAVE_AVCALL
+#include "avcall.h"
 #endif
 
-
 #define MSGBUFLEN 2048
-static char msgBuffer[MSGBUFLEN];
+
+QMutex functiondefmutex;
 
 MFunctionDef::MFunctionDef() {
   functionCompiled = false;
   timeStamp = 0;
-  allCode = NULL;
+  //  allCode = NULL;
   localFunction = false;
-  nextFunction = NULL;
-  prevFunction = NULL;
   pcodeFunction = false;
 }
 
 MFunctionDef::~MFunctionDef() {
-  if (nextFunction != NULL)
-    delete nextFunction;
 }
 
 int MFunctionDef::inputArgCount() {
@@ -78,6 +66,7 @@ int MFunctionDef::outputArgCount() {
 
 void MFunctionDef::printMe(Interpreter*eval) {
   stringVector tmp;
+  char msgBuffer[MSGBUFLEN];
   snprintf(msgBuffer,MSGBUFLEN,"Function name:%s\n",name.c_str());
   eval->outputMessage(msgBuffer);
   eval->outputMessage("Function class: Compiled M function\n");
@@ -111,16 +100,6 @@ ArrayVector MFunctionDef::evaluateFunction(Interpreter *walker,
   context = walker->getContext();
   context->pushScope(name);
   walker->pushDebug(fileName,name);
-  // Push our local functions onto the function scope
-  MFunctionDef *cp;
-  // Walk up until we get to the head of the list
-  cp = this;
-  while (cp->prevFunction != NULL) cp = cp->prevFunction;
-  cp = cp->nextFunction;
-  while (cp != NULL) {
-    context->insertFunctionLocally((FuncPtr) cp);
-    cp = cp->nextFunction;
-  }
   // When the function is called, the number of inputs is
   // at sometimes less than the number of arguments requested.
   // Check the argument count.  If this is a non-varargin
@@ -237,6 +216,21 @@ ArrayVector MFunctionDef::evaluateFunction(Interpreter *walker,
 	for (int i=0;i<toFill;i++)
 	  outputs[explicitCount+i] = dp[i];
       }
+      // Special case - nargout = 0, only variable outputs from function
+      if ((nargout == 0) && (explicitCount == 0)) {
+	Array varargout, *ptr;
+	// Yes, get a pointer to the "vargout" variable that should be defined
+	ptr = context->lookupVariableLocally("varargout");
+	if (ptr) {
+	  varargout = *ptr;
+	  if (varargout.dataClass() != FM_CELL_ARRAY)
+	    throw Exception("The special variable 'varargout' was not defined as a cell-array");
+	  // Get the data pointer
+	  const Array *dp = ((const Array*) varargout.getDataPointer());
+	  if (varargout.getLength() > 0)
+	    outputs << dp[0];
+	}	
+      }
     }
     // Check for arguments that were passed by reference, and 
     // update their values.
@@ -289,35 +283,35 @@ stringVector IdentifierList(tree t) {
   return retval;
 }
 
-MFunctionDef* ConvertParseTreeToMFunctionDef(tree t, string fileName) {
-  MFunctionDef *fp = new MFunctionDef;
-  fp->returnVals = IdentifierList(t.first());
-  fp->name = t.second().text();
-  fp->arguments = IdentifierList(t.child(2));
-  fp->code = t.child(3);
-  fp->fileName = fileName;
-  return fp;
-}
+//MFunctionDef* ConvertParseTreeToMFunctionDef(tree t, string fileName) {
+//  MFunctionDef *fp = new MFunctionDef;
+//  fp->returnVals = IdentifierList(t.first());
+//  fp->name = t.second().text();
+//  fp->arguments = IdentifierList(t.child(2));
+//  fp->code = t.child(3);
+//  fp->fileName = fileName;
+//  return fp;
+//}
+//
+//MFunctionDef* ConvertParseTreeToMFunctionDefs(treeVector t, 
+//					      string fileName) {
+//  MFunctionDef* last = NULL;
+//  for (int index = t.size()-1;index>=0;index--) {
+//    MFunctionDef* rp = ConvertParseTreeToMFunctionDef(t[index],fileName);
+//    if (index>0)
+//      rp->localFunction = true;
+//    else
+//      rp->localFunction = false;
+//    rp->nextFunction = last;
+//    if (last)
+//      last->prevFunction = rp;
+//    last = rp;
+//  }
+//  return last;
+//}
 
-MFunctionDef* ConvertParseTreeToMFunctionDefs(treeVector t, 
-					      string fileName) {
-  MFunctionDef* last = NULL;
-  for (int index = t.size()-1;index>=0;index--) {
-    MFunctionDef* rp = ConvertParseTreeToMFunctionDef(t[index],fileName);
-    if (index>0)
-      rp->localFunction = true;
-    else
-      rp->localFunction = false;
-    rp->nextFunction = last;
-    if (last)
-      last->prevFunction = rp;
-    last = rp;
-  }
-  return last;
-}
-  
 // Compile the function...
-bool MFunctionDef::updateCode() {
+bool MFunctionDef::updateCode(Interpreter *m_eval) {
   if (localFunction) return false;
   if (pcodeFunction) return false;
   // First, stat the file to get its time stamp
@@ -336,6 +330,7 @@ bool MFunctionDef::updateCode() {
     helpText.clear();
     char buffer[1000];
     while (!feof(fp) && commentsOnly) {
+      buffer[0] = 0;
       fgets(buffer,1000,fp);
       char *cp;
       cp = buffer;
@@ -345,16 +340,16 @@ bool MFunctionDef::updateCode() {
 	continue;
       if (*cp != '%') 
 	commentsOnly = false;
-      else
-	helpText.push_back(++cp);
+      else {
+	string htext(++cp);
+	if ((htext.size() > 1) && (htext[htext.size()-1] != '\n'))
+	  helpText.push_back(htext + "\n");
+	else
+	  helpText.push_back(htext);
+      }
     }
     if (helpText.size() == 0)
       helpText.push_back(buffer);
-    if (feof(fp)) {
-      functionCompiled = true;
-      code = NULL;
-      return true;
-    }
     rewind(fp);
     try {
       // Read the file into a string
@@ -366,16 +361,27 @@ bool MFunctionDef::updateCode() {
       allCode = pcode;
       if (pcode.is(TOK_FUNCTION_DEFS)) {
 	scriptFlag = false;
-	MFunctionDef *fp = ConvertParseTreeToMFunctionDefs(pcode.children(),
-							   fileName);
-	returnVals = fp->returnVals;
-	code = fp->code;
-	nextFunction = fp->nextFunction;
-	if (nextFunction)
-	  fp->nextFunction->prevFunction = this;
-	prevFunction = false;
+	// Get the main function..
+	tree MainFuncCode = pcode.first();
+	returnVals = IdentifierList(MainFuncCode.first());
+	// The name is mangled by the interpreter...  We ignore the
+	// name as parsed in the function.
+	//	name = MainFuncCode.second().text();
+	arguments = IdentifierList(MainFuncCode.child(2));
+	code = MainFuncCode.child(3);
 	localFunction = false;
-	arguments = fp->arguments;
+	// Process the local functions
+	for (int index = 1;index < pcode.numchildren();index++) {
+	  tree LocalFuncCode = pcode.child(index);
+	  MFunctionDef *fp = new MFunctionDef;
+	  fp->localFunction = true;
+	  fp->returnVals = IdentifierList(LocalFuncCode.first());
+	  fp->name = fileName + ":Local:" + LocalFuncCode.second().text();
+	  fp->arguments = IdentifierList(LocalFuncCode.child(2));
+	  fp->code = LocalFuncCode.child(3);
+	  fp->fileName = fileName;
+	  m_eval->getContext()->insertFunction(fp,temporaryFlag);
+	}
 	functionCompiled = true;
       } else {
 	scriptFlag = true;
@@ -441,7 +447,7 @@ unsigned MFunctionDef::ClosestLine(unsigned line) {
 }
 
 void MFunctionDef::SetBreakpoint(int bpline, bool enable) {
-  qDebug() << "Set bp called for line " << bpline << " enable is " << enable << " for function :" << QString::fromStdString(name) << "\r\n";
+  //  qDebug() << "Set bp called for line " << bpline << " enable is " << enable << " for function :" << QString::fromStdString(name) << "\r\n";
   if (!SetTreeBreakPoint(allCode,bpline,enable)) 
     throw Exception(string("Unable to modify a breakpoint at line ") + 
 		    bpline + string(" of routine ") + name);
@@ -472,17 +478,8 @@ MFunctionDef* ThawMFunction(Serialize *s) {
   t->helpText = s->getStringVector();
   t->allCode = ThawTree(s);
   if (t->allCode.is(TOK_FUNCTION_DEFS)) {
-    MFunctionDef *fp = ConvertParseTreeToMFunctionDefs(t->allCode.children(),
-						       t->fileName);
-    t->returnVals = fp->returnVals;
-    t->code = fp->code;
-    t->nextFunction = fp->nextFunction;
-    if (t->nextFunction)
-      fp->nextFunction->prevFunction = t;
-    t->prevFunction = false;
-    t->localFunction = false;
-    t->arguments = fp->arguments;
-    t->functionCompiled = true;
+    tree MainFuncCode = t->allCode.first();
+    t->code = MainFuncCode.child(3);
   } else {
     t->scriptFlag = true;
     t->functionCompiled = true;
@@ -507,6 +504,7 @@ int BuiltInFunctionDef::outputArgCount() {
 
 void BuiltInFunctionDef::printMe(Interpreter *eval) {
   stringVector tmp;
+  char msgBuffer[MSGBUFLEN];
   snprintf(msgBuffer,MSGBUFLEN," Function name:%s\n",name.c_str());
   eval->outputMessage(msgBuffer);
   eval->outputMessage(" Function class: Built in\n");
@@ -566,6 +564,23 @@ void SpecialFunctionDef::printMe(Interpreter *eval) {
 FunctionDef::FunctionDef() {
   scriptFlag = false;
   graphicsFunction = false;
+  temporaryFlag = false;
+  refcount = 0;
+}
+
+void FunctionDef::lock() {
+  QMutexLocker lockit(&functiondefmutex);
+  refcount++;
+}
+
+void FunctionDef::unlock() {
+  QMutexLocker lockit(&functiondefmutex);
+  refcount--;
+}
+
+bool FunctionDef::referenced() {
+  QMutexLocker lockit(&functiondefmutex);
+  return (refcount>0);
 }
 
 FunctionDef::~FunctionDef() {
@@ -708,7 +723,7 @@ ArrayVector ImportedFunctionDef::evaluateFunction(Interpreter *walker,
 	values[i] = inputs[i].getReadWriteDataPointer();
       }
     } else {
-      refPointers[ptr] = inputs[i].getContentsAsCString();
+      refPointers[ptr] = strdup(inputs[i].getContentsAsString().c_str());
       values[i] = &refPointers[ptr];
       ptr++;
     }
