@@ -25,7 +25,10 @@
 #include "DynLib.hpp"
 #include "Serialize.hpp"
 #include "mex.h"
-#include <sys/stat.h>
+#include <QSharedData>
+#include <string>
+
+using namespace std;
 
 typedef enum {
   FM_M_FUNCTION,
@@ -56,16 +59,25 @@ typedef ArrayVector (*SpecialFuncPtr) (int,const ArrayVector&,Interpreter*);
  * a well defined number of output arguments, and some means of 
  * being evaluated.
  */
+
 class FunctionDef {
 public:
   /**
    * The name of the function - must follow identifier rules.
    */
   std::string name;
+  /**
+   * The reference count for this functiondef
+   */
+  int refcount;
   /**
    * Is this a script?
    */
   bool scriptFlag;
+  /**
+   * Is this a temporary function?
+   */
+  bool temporaryFlag;
   /**
    * If a function requires access to the graphics subsystem,
    * it must declare itself as such.  Such functions are executed
@@ -86,6 +98,18 @@ public:
    * The virtual destructor
    */
   virtual ~FunctionDef();
+  /**
+   * Increment the reference count
+   */
+  void lock();
+  /**
+   * Decrement the reference count
+   */
+  void unlock();
+  /**
+   * Returns true if the current function has references
+   */
+  bool referenced();
   /**
    * The type of the function (FM_M_FUNCTION, FM_BUILT_IN_FUNCTION,
    * FM_SPECIAL_FUNCTION, FM_IMPORTED_FUNCTION).
@@ -112,13 +136,9 @@ public:
    * function will be called at least once before evaluateFunction is called.
    * Return true if the updateCode call did anything.
    */
-  virtual bool updateCode() {return false;}
+  virtual bool updateCode(Interpreter *) {return false;}
 };
 
-typedef FunctionDef* FunctionDefPtr;
-
-class MFunctionDef;
-
 /**
  * An MFunctionDef is a FunctionDef for an interpreted function.  The
  * function pointer stores the name of the file where the function is
@@ -143,9 +163,6 @@ public:
    * function is contained in this AST, not the function declaration itself).
    */
   tree code;
-  /**
-   * The AST for all of the code in the file.
-   */
   tree allCode;
   /**
    * Flag to indicate if the function has been compiled.
@@ -160,19 +177,7 @@ public:
    */
   time_t timeStamp;
   /**
-   * For some function files, there are multiple functions defined in
-   * a single file.  The subsequent functions are local to the function
-   * scope of the main function, and override global functions inside
-   * the body of the current function (they are essentially hidden inside
-   * the scope of the current function).  These functions are parsed
-   * and form a linked list of function definitions, with the main function
-   * at the head of the list.
-   */
-  MFunctionDef *nextFunction;
-  MFunctionDef *prevFunction;
-  /**
-   * Set to true for all of the localFunctions.  False for the head of the
-   * linked list.
+   * Set to true for all of the localFunctions.  
    */
   bool localFunction;
   bool pcodeFunction;
@@ -222,7 +227,7 @@ public:
    * parsing of the file (i.e., it cannot be classified as either a
    * script or a function definition).
    */
-  virtual bool updateCode();
+  virtual bool updateCode(Interpreter *m_eval);
   /**
    * Activate/Deactivate a breakpoint
    */
@@ -285,8 +290,6 @@ public:
   virtual ArrayVector evaluateFunction(Interpreter *, ArrayVector &, int);
 };
 
-typedef FunctionDef* FuncPtr;
-
 class SpecialFunctionDef : public FunctionDef {
 public:
   /**
@@ -440,4 +443,64 @@ public:
    */
   virtual ArrayVector evaluateFunction(Interpreter *, ArrayVector& , int);    
 };
+
+// This used to be a simple typedef to a pointer of a functiondef
+// Now, it is a reference counted class.
+class FuncPtr {
+private:
+  FunctionDef* d;
+public:
+  FuncPtr() : d(NULL) {}
+  ~FuncPtr() {
+    if (d) {
+      d->unlock();
+      if (!d->referenced()) delete d;
+    }
+  }
+  FuncPtr(SpecialFunctionDef* ptr) {
+    d = ptr;
+    if (d)
+      d->lock();
+  }
+  FuncPtr(BuiltInFunctionDef* ptr) {
+    d = ptr;
+    if (d)
+      d->lock();
+  }
+  FuncPtr(FunctionDef* ptr) {
+    d = ptr;
+    if (d)
+      d->lock();
+  }
+  FuncPtr(const FuncPtr &copy) {
+    d = copy.d;
+    if (d)
+      d->lock();
+  }
+  FuncPtr& operator=(const FuncPtr &copy) {
+    if (copy.d == d)
+      return *this;
+    if (d) {
+      d->unlock();
+      if (!d->referenced()) delete d;
+    }
+    d = copy.d;
+    if (d)
+      d->lock();
+    return *this;
+  }
+  FunctionDef* operator->() const {
+    return d;
+  }
+  FunctionDef& operator*() const {
+    return *d;
+  }
+  bool operator!() const {
+    return (d == NULL);
+  }
+  operator FunctionDef* () const {return d;}
+  operator MFunctionDef* () const {return ((MFunctionDef*)d);}
+};
+
+
 #endif
