@@ -1010,9 +1010,9 @@ template void getNDimSubsetNumericDispatchReal<Array>(int colonIndex,
 						      int outDimsInt[maxDims], 
 						      int srcDimsInt[maxDims], 
 						      constIndexPtr* indx, int L);
-template void getNDimSubsetNumericDispatchReal<FunctionDefPtr>(int colonIndex, 
-							       const FunctionDefPtr* srcptr, 
-							       FunctionDefPtr* destptr, 
+template void getNDimSubsetNumericDispatchReal<FuncPtr>(int colonIndex, 
+							       const FuncPtr* srcptr, 
+							       FuncPtr* destptr, 
 							       int outDimsInt[maxDims], 
 							       int srcDimsInt[maxDims], 
 							       constIndexPtr* indx, int L);
@@ -1117,9 +1117,9 @@ template void setNDimSubsetNumericDispatchReal<Array>(int colonIndex,
 						      int outDimsInt[maxDims], 
 						      int srcDimsInt[maxDims], 
 						      constIndexPtr* indx, int L,int advance);
-template void setNDimSubsetNumericDispatchReal<FunctionDefPtr>(int colonIndex, 
-							       FunctionDefPtr* srcptr, 
-							       const FunctionDefPtr* destptr, 
+template void setNDimSubsetNumericDispatchReal<FuncPtr>(int colonIndex, 
+							       FuncPtr* srcptr, 
+							       const FuncPtr* destptr, 
 							       int outDimsInt[maxDims], 
 							       int srcDimsInt[maxDims], 
 							       constIndexPtr* indx, int L,int advance);
