@@ -27,7 +27,7 @@ int GetNominalWidthInteger(const T*array, int count) {
   int maxdigit = 0;
   for (int i=0;i<count;i++) {
     memset(buffer,0,MSGBUFLEN);
-    sprintf(buffer,"%lld",array[i]);
+    sprintf(buffer,"%lld",(int64)array[i]);
     int j = maxdigit;
     while (buffer[j] && j>=maxdigit)
       j++;
@@ -39,7 +39,7 @@ int GetNominalWidthInteger(const T*array, int count) {
 template <class T>
 void PrintIntegers(const T*array, int count, int width) {
   for (int i=0;i<count;i++)
-    printf("%*lld\n",width,array[i]);
+    printf("%*lld\n",width,(int64)array[i]);
 }
   
 template <class T>
@@ -244,7 +244,7 @@ void emitFormattedElement(Interpreter* io, char *msgBuffer, ArrayFormat format,
     if (ap == NULL)
       io->outputMessage("[]");
     else
-      ap[num].summarizeCellEntry();
+      ap[num].summarizeCellEntry(io);
     io->outputMessage("  ");
     break;
   }
@@ -417,12 +417,12 @@ void PrintSheet(ArrayFormat format, Interpreter*io, int rows, int columns,
   // the terminal width
   int colsPerPage;
   if (aclass != FM_STRING)
-    colsPerPage = (int) floor((termWidth-1)/((float) format.width + 3));
+    colsPerPage = (int) floor((termWidth-1)/((double) format.width + 3));
   else
     colsPerPage = termWidth;
   colsPerPage = (colsPerPage < 1) ? 1 : colsPerPage;
   int pageCount;
-  pageCount = (int) ceil(columns/((float)colsPerPage));
+  pageCount = (int) ceil(columns/((double)colsPerPage));
   for (int k=0;k<pageCount;k++) {
     int colsInThisPage;
     colsInThisPage = columns - colsPerPage*k;
@@ -485,7 +485,7 @@ void PrintArrayClassic(Array A, int printlimit, Interpreter* io,
 	io->outputMessage("    ");
 	io->outputMessage(A.fieldNames().at(n).c_str());
 	io->outputMessage(": ");
-	ap[n].summarizeCellEntry();
+	ap[n].summarizeCellEntry(io);
 	io->outputMessage("\n");
       }
     } else {
@@ -535,6 +535,8 @@ void PrintArrayClassic(Array A, int printlimit, Interpreter* io,
 
 
 string ArrayToPrintableString(const Array& a) {
+  if (a.isEmpty())
+    return string("[]");
   if (a.sparse())
     return string("");
   if (a.isString())
