@@ -24,13 +24,12 @@
  * A Scope is a combination of a variable hashtable and a function hashtable.
  */
 #include <string>
+#include <QMutex>
 
 #include "Array.hpp"
-#include "FunctionDef.hpp"
 #include "SymbolTable.hpp"
 
 typedef SymbolTable<Array> VariableTable;
-typedef SymbolTable<FuncPtr> CodeTable;
 
 /**
  * A Scope is a collection of functions and variables all visible
@@ -39,6 +38,12 @@ typedef SymbolTable<FuncPtr> CodeTable;
  * relevant to the current scope.
  */
 class Scope {
+  /**
+   * This is a mutex to protect the scope when multiple threads
+   * have access to the scope.  For all scopes (except the global
+   * one, this mutex is unused.
+   */
+  QMutex *mutex;
   /**
    * This is the hash-table of Array pointers that forms the
    * symbol table.  Each variable has a name associated with
@@ -47,12 +52,6 @@ class Scope {
    * destructing them when destroyed.
    */
   VariableTable symTab;
-  /**
-   * This is a hash-table of function pointers.  The Scope does
-   * _not_ own the function pointers, and it must not destroy
-   * them when destructed.
-   */
-  CodeTable codeTab;
   /**
    * The name of the scope.
    */
@@ -93,7 +92,29 @@ public:
    * Construct a scope with the given name.
    */
   Scope(std::string scopeName) : name(scopeName), loopLevel(0), 
-    anyPersistents(false), anyGlobals(false) {}
+				 anyPersistents(false), anyGlobals(false),
+				 mutex(NULL)  {}
+  /**
+   * Lock the scope's mutex
+   */
+  inline void lock() {
+    if (mutex) mutex->lock();
+  }
+  /**
+   * Unlock the scope's mutex
+   */
+  inline void unlock() {
+    if (mutex) mutex->unlock();
+  }
+  /**
+   * Initialize the scope's mutex - only needed in Global
+   * scope.
+   */
+  inline void mutexSetup() {
+    if (mutex) 
+      delete mutex;
+    mutex = new QMutex(QMutex::Recursive);
+  }
   /**
    * Insert a variable with the given name.  If the variable
    * already exists in the Scope, then the previous definition
@@ -102,31 +123,6 @@ public:
   inline void insertVariable(const std::string& varName, const Array& val) {
     symTab.insertSymbol(varName,val);
   }
-  /**
-   * Insert a function pointer into the current scope.  The name of
-   * of the function is encoded in the FuncPtr.
-   */
-  inline void insertFunction(FuncPtr a) {
-    codeTab.insertSymbol(a->name,a);
-  }
-  /**
-   * Delete a function from the current scope.
-   */
-  inline void deleteFunction(const std::string& funcName) {
-    codeTab.deleteSymbol(funcName);
-  }
-  /**
-   * Lookup a function.  Return true if the function is defined, and
-   * assigns the value of the function pointer to the second argument.
-   */
-  inline bool lookupFunction(std::string funcName, FuncPtr& val) {
-    FuncPtr* ret = codeTab.findSymbol(funcName);
-    if (ret) {
-      val = *ret;
-      return true;
-    }
-    return false;
-  }
   /**
    * Lookup a variable.  Return a pointer to the variable in the symbol 
    * table if found and NULL otherwise.  Different than lookupFunction
@@ -242,24 +238,9 @@ public:
    * Get a list of all possible completions of the given
    * string.
    */
-  inline stringVector getCompletions(const std::string& prefix) {
-    stringVector codecompletions;
-    stringVector varcompletions;
-    codecompletions = codeTab.getCompletions(prefix);
-    varcompletions = symTab.getCompletions(prefix);
-    codecompletions.insert(codecompletions.end(),
-			   varcompletions.begin(),
-			   varcompletions.end());
-    return codecompletions;
+   inline stringVector getCompletions(const std::string& prefix) {
+     return symTab.getCompletions(prefix);
   }
-  /**
-   * Returns a list of all functions
-   *
-   */
-  inline stringVector listAllFunctions() {
-    return codeTab.getCompletions("");
-  }
-
   /**
    * Returns a list of all currently defined variables
    * in the active scope.
