@@ -126,7 +126,7 @@ void* SparseScalarMultiply(Class dclass, const void *ap, int rows, int cols,
 void* SparseSolveLinEq(Class dclass, int Arows, int Acols, const void *Ap,
 		       int Brows, int Bcols, const void *Bp);
 ArrayVector SparseLUDecompose(int nargout, Array A);
-ArrayVector SparseEigDecompose(int nargout, Array A, int k, char* whichFlag);
+ArrayVector SparseEigDecompose(int nargout, Array A, int k, string whichFlag);
 ArrayVector SparseEigDecomposeShifted(int nargout, Array A, int k, double shift[2]);
 // Test 81
 void* SparseOnesFunc(Class dclass, int Arows, int Acols, const void *Ap);
