@@ -208,6 +208,8 @@ static std::string TrimPrint(double val, bool scientificNotation) {
   }
 }
 
+const int MAX_TICK_COUNT = 1000;
+
 double tlog(double x) {
   if (x>0) 
     return log10(x);
@@ -225,7 +227,7 @@ void FormatAxisManual(double t1, double t2, int tickcount,
   double delt = (t2-t1)/tickcount;
   int n = (int) ceil(log10(delt));
   double rdelt = delt/pow(10.0,(double)n);
-  int p = (int) floor(log2(rdelt));
+  int p = (int) floor(log(rdelt)/M_LN2);
   double tDelt = pow(10.0,(double) n)*pow(2.0,(double) p);
   if (isLogarithmic)
     tDelt = ceil(tDelt);
@@ -242,12 +244,20 @@ void FormatAxisManual(double t1, double t2, int tickcount,
   const double *dp = (const double *) trange.getDataPointer();
   for (int i=0;i<tCount;i++) {
     double tloc = dp[i];
-    tickLocations.push_back(tloc);
+    if (!isLogarithmic)
+      tickLocations.push_back(tloc);
+    else
+      tickLocations.push_back(pow(10.0,tloc));
     if (tloc != 0.0)
       exponentialForm |= (fabs(log10(fabs(tloc))) >= 4.0);
   }
-  for (int i=0;i<tCount;i++) 
-    tlabels.push_back(TrimPrint(tBegin+i*tDelt,exponentialForm));
+  for (int i=0;i<tCount;i++) {
+    double tloc = tBegin+i*tDelt;
+    if (!isLogarithmic)
+      tlabels.push_back(TrimPrint(tloc,exponentialForm));
+    else
+      tlabels.push_back(TrimPrint(pow(10.0,tloc),true));
+  }
 }
   
 void FormatAxisAuto(double tMin, double tMax, int tickcount,
@@ -260,7 +270,7 @@ void FormatAxisAuto(double tMin, double tMax, int tickcount,
   double delt = (tMax-tMin)/(tickcount);
   int n = (int) ceil(log10(delt));
   double rdelt = delt/pow(10.0,(double)n);
-  int p = (int) floor(log2(rdelt));
+  int p = (int) floor(log(rdelt)/M_LN2);
   double tDelt = pow(10.0,(double) n)*pow(2.0,(double) p);
   if (isLogarithmic) 
     tDelt = ceil(tDelt);
@@ -1535,7 +1545,7 @@ int HandleAxis::GetTickCount(RenderEngine &gc,
 }
 
 void HandleAxis::RecalculateTicks() {
-  QImage img(1,1,QImage::Format_RGB32);
+  QPixmap img(1,1);
   QPainter pnt(&img);
   HandleFigure *fig = GetParentFigure();
   unsigned width = fig->GetWidth();
@@ -2095,7 +2105,7 @@ void HandleAxis::DrawTickMarks(RenderEngine &gc) {
 	double t1 = xticks[i];
 	double t2 = xticks[i+1];
 	int n = 2;
-	while ((t1*n) < t2) {
+	while (((t1*n) < t2) && (n < MAX_TICK_COUNT)) {
 	  minorticks.push_back(MapX(n*t1));
 	  n++;
 	}
@@ -2121,7 +2131,7 @@ void HandleAxis::DrawTickMarks(RenderEngine &gc) {
 	double t1 = yticks[i];
 	double t2 = yticks[i+1];
 	int n = 2;
-	while ((t1*n) < t2) {
+	while (((t1*n) < t2) && (n < MAX_TICK_COUNT)) {
 	  minorticks.push_back(MapY(n*t1));
 	  n++;
 	}
@@ -2147,7 +2157,7 @@ void HandleAxis::DrawTickMarks(RenderEngine &gc) {
 	double t1 = zticks[i];
 	double t2 = zticks[i+1];
 	int n = 2;
-	while ((t1*n) < t2) {
+	while (((t1*n) < t2) && (n < MAX_TICK_COUNT)) {
 	  minorticks.push_back(MapZ(n*t1));
 	  n++;
 	}
@@ -2359,6 +2369,10 @@ void HandleAxis::DrawAxisLabels(RenderEngine& gc) {
   std::string ydir(StringPropertyLookup("ydir"));
   SetStringDefault("xdir","normal");
   SetStringDefault("ydir","normal");
+  std::string xscale(StringPropertyLookup("xscale"));
+  std::string yscale(StringPropertyLookup("yscale"));
+  SetStringDefault("xscale","linear");
+  SetStringDefault("yscale","linear");
   if (xvisible) {
     lbl = (HPHandles*) LookupProperty("xlabel");
     if (!lbl->Data().empty()) {
@@ -2387,8 +2401,9 @@ void HandleAxis::DrawAxisLabels(RenderEngine& gc) {
   }      
   SetupProjection(gc);
   SetStringDefault("xdir",xdir);
-
   SetStringDefault("ydir",ydir);
+  SetStringDefault("xscale",xscale);
+  SetStringDefault("yscale",yscale);
 }
 
 void HandleAxis::DrawChildren(RenderEngine& gc) {
