@@ -29,6 +29,7 @@
 
 class BaseFigureQt : public QWidget {
   HandleFigure *hfig;
+  QPixmap backStore;
 public:
   BaseFigureQt(QWidget *parent, HandleFigure *fig);
   void paintEvent(QPaintEvent *e);
@@ -49,13 +50,36 @@ void BaseFigureQt::resizeEvent(QResizeEvent *e) {
   //  qDebug() << "resize " << width() << " " << height() << "\r\n";
   hfig->resizeGL(qMax(8,width()),
   		 qMax(8,height()));
+  backStore = QPixmap(width(),height());
+}
+
+static bool enableRepaint = false;
+
+void GfxEnableRepaint() {
+  enableRepaint = true;
+  DoDrawNow();
+}
+
+void GfxDisableRepaint() {
+  enableRepaint = false;
 }
 
 void BaseFigureQt::paintEvent(QPaintEvent *e) {
-  QPainter pnt(this);
-  QTRenderEngine gc(&pnt,0,0,width(),height());
-  hfig->PaintMe(gc);
-  QWidget::paintEvent(e);
+  if (enableRepaint && hfig->ParentWindow()->isDirty()) {
+    {
+      // enableRepaint is true, and the background is dirty - update
+      // the backing store, and then redraw it.
+      QPainter pnt(&backStore);
+      QTRenderEngine gc(&pnt,0,0,width(),height());
+      hfig->PaintMe(gc);
+      hfig->ParentWindow()->markClean();
+    }
+    QPainter pnt2(this);
+    pnt2.drawPixmap(0,0,backStore);
+  } else {
+    QPainter pnt(this);
+    pnt.drawPixmap(e->rect(),backStore);
+  }
 }
 
 BaseFigureQt::BaseFigureQt(QWidget *parent, HandleFigure *fig) : 
@@ -176,6 +200,7 @@ HandleWindow::HandleWindow(unsigned ahandle) : QMainWindow() {
   resize(600,400);
   initialized = true;
   mode = normal_mode;
+  dirty = false;
 }
 
 
@@ -233,11 +258,19 @@ void HandleWindow::camRotate(bool active) {
 
 void HandleWindow::save() {
   QString fn = QFileDialog::getSaveFileName();
-  HPrintFunction(0,singleArrayVector(Array::stringConstructor(fn.toStdString())));
+  try {
+    HPrintFunction(0,singleArrayVector(Array::stringConstructor(fn.toStdString())));
+  } catch(Exception &e) {
+    QMessageBox::critical(0,"Critical Error in Save",QString::fromStdString(e.getMessageCopy()));
+  }
 }
 
 void HandleWindow::copy() {
-  HCopyFunction(0,ArrayVector());
+  try {
+    HCopyFunction(0,ArrayVector());
+  } catch(Exception &e) {
+    QMessageBox::critical(0,"Critical Error in Copy",QString::fromStdString(e.getMessageCopy()));
+  }
 }
 
 void HandleWindow::createActions() {
@@ -584,6 +617,7 @@ void HandleWindow::mouseReleaseEvent(QMouseEvent * e) {
 
 void HandleWindow::UpdateState() {
   if (!initialized) return;
+    dirty = true;
   //  HPTwoVector *htv = (HPTwoVector*) hfig->LookupProperty("figsize");
   //  qtchild->resize((int)(htv->Data()[0]),(int)(htv->Data()[1]));
   //  qtchild->updateGeometry();
@@ -606,6 +640,6 @@ void HandleWindow::UpdateState() {
   // 	glchild->setGeometry(0,0,1,1);
   //  layout->setCurrentWidget(qtchild);
   //     }
-  update();
+    update();
   //   }
 }
