@@ -54,6 +54,7 @@
 // ----------------------------------------------------------------------------
 
 KeyManager::KeyManager()  {
+  keypresswait = false;
   cutbuf = "";
   linelen = 1000;
   ntotal = 0;
@@ -68,7 +69,7 @@ KeyManager::KeyManager()  {
   enteredLinesEmpty = true;
   ReplacePrompt("");
   loopactive = 0;
-  lineData = new char[LINELEN];
+  lineData = "";
   ResetLineBuffer();
   context = NULL;
   QSettings settings("FreeMat", "FreeMat");
@@ -151,27 +152,25 @@ int KeyManager::DisplayedStringWidth(string s, int nc, int aterm_curpos) {
 }
 
 void KeyManager::InsertString(int pos, string s) {
-  int len = s.size();
-  for (int i=4096-len;i>pos;i--)
-    lineData[i] = lineData[i-len];
-  const char* sptr = s.c_str();
-  for (int i=0;i<len;i++)
-    lineData[pos+i] = sptr[i];
+  lineData.insert(pos,s);
 }
 
 void KeyManager::InsertCharacter(int pos, char c) {
-  for (int i=4095;i>pos;i--)
-    lineData[i] = lineData[i-1];
-  lineData[pos] = c;
+  lineData.insert(pos,1,c);
 }
 
 void KeyManager::EraseCharacters(int pos, int cnt) {
-  for (int i=pos+cnt;i<4096;i++)
-    lineData[i-cnt] = lineData[i];
+  lineData.erase(pos,cnt);
 }
 
 void KeyManager::SetCharacter(int pos, char c) {
-  lineData[pos] = c;
+  if (pos < lineData.size())
+    lineData[pos] = c;
+  else {
+    int topad = (pos-lineData.size()+1);
+    lineData.append(topad,' ');
+    lineData[pos] = c;
+  }
 }
 
 int KeyManager::BuffCurposToTermCurpos(int n) {
@@ -347,7 +346,7 @@ void KeyManager::AddCharToLine(char c) {
      * Redraw the line from the cursor position to the end of the line,
      * and move the cursor to just after the added character.
      */
-    OutputString(string(lineData+sbuff_curpos), '\0');
+    OutputString(string(lineData,sbuff_curpos), '\0');
     SetTermCurpos(sterm_curpos + width);
     /*
      * Are we overwriting an existing character?
@@ -370,7 +369,7 @@ void KeyManager::AddCharToLine(char c) {
      * with spaces.
      */
     if(old_width > width) {
-      OutputString(string(lineData+sbuff_curpos), '\0');
+      OutputString(string(lineData,sbuff_curpos), '\0');
       /*
        * Clear to the end of the terminal.
        */
@@ -389,7 +388,7 @@ void KeyManager::AddCharToLine(char c) {
        * Redraw the line from the cursor position to the end of the line,
        * and move the cursor to just after the added character.
        */
-      OutputString(string(lineData+sbuff_curpos), '\0');
+      OutputString(string(lineData,sbuff_curpos), '\0');
       SetTermCurpos(sterm_curpos + width);
       buff_curpos++;
       /*
@@ -519,14 +518,14 @@ void KeyManager::ResetLineBuffer() {
   term_curpos = 0;
   term_len = 0;
   insert_curpos = 0;
-  memset(lineData,0,4096);
+  lineData = "";
 }
 
 void KeyManager::NewLine() {
   AddHistory(lineData);
   PlaceCursor(ntotal);
   emit OutputRawString("\r\n");
-  emit ExecuteLine(string(lineData) + "\n");
+  emit ExecuteLine(lineData + "\n");
   ReplacePrompt("");
   ResetLineBuffer();
   DisplayPrompt();
@@ -537,10 +536,6 @@ void KeyManager::Ready() {
   Redisplay();
 }
 
-void KeyManager::RegisterInterrupt() {
-  sigInterrupt(0);
-}
-
 void KeyManager::CursorLeft() {
   PlaceCursor(buff_curpos-1);
 }
@@ -594,7 +589,7 @@ void KeyManager::DeleteChars(int nc, int cut) {
   /*
    * Redraw the remaining characters following the cursor.
    */
-  OutputString(string(lineData+buff_curpos), '\0');
+  OutputString(string(lineData,buff_curpos), '\0');
   /*
    * Clear to the end of the terminal.
    */
@@ -610,9 +605,9 @@ void KeyManager::EndOfLine() {
 }
 
 void KeyManager::KillLine() {
-  cutbuf = string(lineData+buff_curpos);
+  cutbuf = string(lineData,buff_curpos);
   ntotal = buff_curpos;
-  memset(lineData+ntotal,0,4096-ntotal);
+  lineData.erase(ntotal);
   TruncateDisplay();
   PlaceCursor(buff_curpos);
 }
@@ -622,7 +617,7 @@ void KeyManager::HistorySearchBackward() {
     SearchPrefix(string(lineData),buff_curpos);
   last_search = keyseq_count;
   HistoryFindBackwards();
-  ntotal = strlen(lineData);
+  ntotal = lineData.size();
   buff_curpos = ntotal;
   Redisplay();
 }
@@ -632,7 +627,7 @@ void KeyManager::HistorySearchForward() {
     SearchPrefix(string(lineData),buff_curpos);
   last_search = keyseq_count;
   HistoryFindForwards();
-  ntotal = strlen(lineData);
+  ntotal = lineData.size();
   buff_curpos = ntotal;
   Redisplay();
 }
@@ -666,8 +661,7 @@ void KeyManager::HistoryFindForwards() {
     ResetLineBuffer();
     return;
   }
-  memset(lineData,0,LINELEN);
-  strcpy(lineData,history[i].c_str());
+  lineData = history[i];
   startsearch = i;
 }
 
@@ -682,8 +676,7 @@ void KeyManager::HistoryFindBackwards() {
     if (!found) i--;
   }
   if (!found) return;
-  memset(lineData,0,LINELEN);
-  strcpy(lineData,history[i].c_str());
+  lineData = history[i];
   startsearch = i;
 }
 
@@ -739,7 +732,7 @@ void KeyManager::AddStringToLine(string s) {
    * Write the modified part of the line to the terminal, then move
    * the terminal cursor to the end of the displayed input string.
    */
-  OutputString(string(lineData+sbuff_curpos), '\0');
+  OutputString(string(lineData,sbuff_curpos), '\0');
   SetTermCurpos(sterm_curpos + term_slen);
 }
 
@@ -919,7 +912,7 @@ void KeyManager::CompleteWord() {
 	 */
 	if(!redisplay) {
 	  TruncateDisplay();
-	  OutputString(string(lineData+buff_pos), '\0');
+	  OutputString(string(lineData,buff_pos), '\0');
 	  PlaceCursor(buff_curpos);
 	  return;
 	};
@@ -939,7 +932,17 @@ void KeyManager::CompleteWord() {
   return;
 }
 
+void KeyManager::getKeyPress() {
+  keypresswait = true;
+  while (keypresswait) 
+    qApp->processEvents();
+}
+
 void KeyManager::OnChar( int c ) {
+  if (keypresswait) {
+    keypresswait = false;
+    return;
+  }
   keyseq_count++;
   switch(c) {
   case KM_BACKSPACE:
@@ -973,8 +976,8 @@ void KeyManager::OnChar( int c ) {
   case KM_CTRLA:
     BeginningOfLine();
     break;
-  case KM_CTRLZ:
-    RegisterInterrupt();
+  case KM_CTRLB:
+    emit RegisterInterrupt();
     break;
   case KM_CTRLE:
     EndOfLine();
@@ -1036,7 +1039,7 @@ void KeyManager::QueueCommand(QString t) {
   QueueString(t);
   AddHistory(t.toStdString());
   emit OutputRawString("\r\n");
-  emit ExecuteLine(string(lineData)+"\n");
+  emit ExecuteLine(lineData+"\n");
   ResetLineBuffer();
   DisplayPrompt();
 }
@@ -1070,10 +1073,15 @@ void KeyManager::RegisterTerm(QObject* term) {
   connect(this,SIGNAL(ClearEOD()),term,SLOT(ClearEOD()));
   connect(this,SIGNAL(MoveBOL()),term,SLOT(MoveBOL()));
   connect(this,SIGNAL(OutputRawString(string)),term,SLOT(OutputRawString(string)));
+  connect(this,SIGNAL(ClearDisplay()),term,SLOT(ClearDisplay()));
   connect(term,SIGNAL(OnChar(int)),this,SLOT(OnChar(int)));
   connect(term,SIGNAL(SetTextWidth(int)),this,SLOT(SetTermWidth(int)));  
 }
 
+void KeyManager::ClearDisplayCommand() {
+  emit ClearDisplay();
+}
+
 void KeyManager::ContinueAction() {
   emit ExecuteLine("return\n");
 }
@@ -1170,15 +1178,9 @@ vector<string> KeyManager::GetCompletions(string line,
    */
   if (!context) return completions;
   if (start[-1] != '\'') {
-    vector<string> local_completions;
-    vector<string> global_completions;
-    int i;
-    local_completions = context->getCurrentScope()->getCompletions(string(start));
-    global_completions = context->getGlobalScope()->getCompletions(string(start));
-    for (i=0;i<local_completions.size();i++)
+    vector<string> local_completions(context->getCompletions(string(start)));
+    for (int i=0;i<local_completions.size();i++)
       completions.push_back(local_completions[i]);
-    for (i=0;i<global_completions.size();i++)
-      completions.push_back(global_completions[i]);
   }
   stringVector comp(GetCompletionList(tmp));
   for (int i=0;i<comp.size();i++)
