@@ -28,6 +28,7 @@
 #include "Context.hpp"
 
 #define KM_CTRLA     0x01
+#define KM_CTRLB     0x02
 #define KM_CTRLZ     0x1A
 #define KM_CTRLD     0x04
 #define KM_CTRLE     0x05
@@ -74,6 +75,7 @@ public:
   void RegisterTerm(QObject* term);
   void SetCompletionContext(Context* ctxt);
   Context* GetCompletionContext();
+  void getKeyPress();
  private:
   Context *context;
   vector<string> GetCompletions(string line, int word_end, 
@@ -110,7 +112,7 @@ public:
   // the text
   vector<string> history;
   // The new line buffer
-  char *lineData;
+  string lineData;
   // The maximum allowed line length
   int linelen;
   // The cut buffer
@@ -140,6 +142,8 @@ public:
   int prompt_len;
   // Are we waiting for input?
   int loopactive;
+  // Are we waiting for a key press
+  bool keypresswait;
 signals:
   void CWDChanged();
   void UpdateCWD();
@@ -150,6 +154,7 @@ signals:
   void ClearEOL();
   void ClearEOD();
   void MoveBOL();
+  void ClearDisplay();
   void OutputRawString(string txt);
   void SendCommand(QString);
   void Interrupt();
@@ -163,14 +168,15 @@ public slots:
   void QueueMultiString(QString);
   void QueueCommand(QString);
   void QueueSilent(QString);
-  void RegisterInterrupt();
   void ContinueAction();
   void StopAction();
   void DbStepAction();
   void DbTraceAction();
   void SetPrompt(string);
+  void ClearDisplayCommand();
 signals:
   void UpdateTermWidth(int);
+  void RegisterInterrupt();
 };
 
 #endif
