@@ -69,6 +69,12 @@ QTTerm::QTTerm(QWidget *parent) :
 QTTerm::~QTTerm() {
 }
 
+void QTTerm::clearSelection() {
+  selectionStart = selectionStop = 0;
+  for (int i=0;i<m_width*m_scrollback;i++)
+    m_history[i].clearSelection();
+}
+
 char* QTTerm::getSelectionText() {
   int start;
   int stop;
@@ -207,11 +213,11 @@ void QTTerm::PaintRectangle(QPainter *paint, QRect rect) {
   paint->eraseRect(rect);
   int col_start, col_stop;
   int row_start, row_stop;
-  col_start = (int)floor(rect.left()/m_char_w);
-  col_stop = (int)ceil(rect.right()/m_char_w);
+  col_start = (int)floor(rect.left()/((float)m_char_w));
+  col_stop = (int)ceil(rect.right()/((float)m_char_w));
   col_stop = qMin(col_stop,m_width-1);
-  row_start = (int)ceil(rect.top()/m_char_h);
-  row_stop = (int)floor(rect.bottom()/m_char_h);
+  row_start = (int)ceil(rect.top()/((float)m_char_h));
+  row_stop = (int)floor(rect.bottom()/((float)m_char_h));
   row_stop = qMin(row_stop,m_height-1);
 
 
@@ -457,6 +463,15 @@ void QTTerm::MoveBOL() {
   SetCursor(0,m_cursor_y);
 }
 
+void QTTerm::ClearDisplay() {
+  for (int i=0;i<m_width*m_scrollback;i++)
+    m_history[i] = tagChar(' ');
+  for (int i=0;i<m_width*m_height;i++)
+    m_surface[i] = tagChar(' ');
+  SetCursor(0,0);
+  m_history_lines = 0;
+  SetupScrollBar(0,0,1,m_height,0);
+}
 
 void QTTerm::ClearEOL() {
   for (int i=m_cursor_x;i<m_width;i++)
