@@ -93,6 +93,7 @@ public:
   void adjustScrollbarPosition();
 
   char *getSelectionText();
+  void clearSelection();
   char *getTextSurface(int &count, int &width);
 
   // The QT interface goes here
@@ -121,6 +122,7 @@ protected:
   void ClearEOL();
   void ClearEOD();
   void MoveBOL();
+  void ClearDisplay();
   void OutputRawString(string txt);
 signals:
  void OnChar(int c);
