@@ -77,6 +77,8 @@ void ApplicationWindow::createActions() {
   saveAct = new QAction(QIcon(":/images/save.png"),"&Save Transcript",this);
   saveAct->setShortcut(Qt::Key_S | Qt::CTRL);
   connect(saveAct,SIGNAL(triggered()),this,SLOT(save()));
+  clearAct = new QAction("&Clear Console",this);
+  connect(clearAct,SIGNAL(triggered()),this,SLOT(clearconsole()));
   quitAct = new QAction(QIcon(":/images/quit.png"),"&Quit",this);
   connect(quitAct,SIGNAL(triggered()),this,SLOT(close()));
   copyAct = new QAction(QIcon(":/images/copy.png"),"&Copy",this);
@@ -106,6 +108,7 @@ void ApplicationWindow::createActions() {
 void ApplicationWindow::createMenus() {
   fileMenu = menuBar()->addMenu("&File");
   fileMenu->addAction(saveAct);
+  fileMenu->addAction(clearAct);
   fileMenu->addAction(quitAct);
   editMenu = menuBar()->addMenu("&Edit");
   editMenu->addAction(copyAct);
@@ -222,6 +225,10 @@ void ApplicationWindow::SetGUITerminal(QTTerm* term) {
   term->show();
 }
 
+void ApplicationWindow::clearconsole() {
+  m_keys->ClearDisplayCommand();
+}
+
 void ApplicationWindow::SetKeyManager(KeyManager *keys) {
   m_keys = keys;
   connect(keys,SIGNAL(SendCommand(QString)),
@@ -229,12 +236,12 @@ void ApplicationWindow::SetKeyManager(KeyManager *keys) {
   connect(m_tool->getHistoryWidget(),SIGNAL(sendCommand(QString)),
  	  keys,SLOT(QueueCommand(QString)));
   connect(m_tool->getFileTool(),SIGNAL(sendCommand(QString)),
- 	  keys,SLOT(QueueString(QString)));
+ 	  keys,SLOT(QueueMultiString(QString)));
   connect(keys,SIGNAL(UpdateCWD()),
 	  m_tool->getFileTool(),SLOT(updateCWD()));
   connect(keys,SIGNAL(UpdateVariables()),
 	  m_tool->getVariablesTool(),SLOT(refresh()));
-  connect(pauseAct,SIGNAL(triggered()),m_keys,SLOT(RegisterInterrupt()));
+  connect(pauseAct,SIGNAL(triggered()),m_keys,SIGNAL(RegisterInterrupt()));
   connect(continueAct,SIGNAL(triggered()),m_keys,SLOT(ContinueAction()));
   connect(stopAct,SIGNAL(triggered()),m_keys,SLOT(StopAction()));
   connect(dbStepAct,SIGNAL(triggered()),m_keys,SLOT(DbStepAction()));
@@ -294,8 +301,10 @@ void ApplicationWindow::paste() {
   if (text.isNull())
     text = cb->text(QClipboard::Clipboard);
   text.replace('\r',"");
-  if (!text.isNull())
+  if (!text.isNull()) {
     m_keys->QueueMultiString(text);
+    m_term->clearSelection();
+  }
 }
 
 void ApplicationWindow::font() {
